@@ -2,14 +2,14 @@
 ;;;; *************************************************************************
 ;;;; FILE IDENTIFICATION
 ;;;;
-;;;; Name:          views.lisp
-;;;; Purpose:       View methods for Hyperobjects
-;;;; Programmer:    Kevin M. Rosenberg
-;;;; Date Started:  Apr 2000
+;;;; Name:     views.lisp
+;;;; Purpose:  View methods for Hyperobjects
+;;;; Author:   Kevin M. Rosenberg
+;;;; Created:  Apr 2000
 ;;;;
-;;;; $Id: views.lisp 8226 2003-11-16 15:49:47Z kevin $
+;;;; $Id: views.lisp 9605 2004-06-13 19:49:18Z kevin $
 ;;;;
-;;;; This file is Copyright (c) 2000-2003 by Kevin M. Rosenberg
+;;;; This file is Copyright (c) 2000-2004 by Kevin M. Rosenberg
 ;;;; *************************************************************************
  
 (in-package #:hyperobject)
@@ -89,10 +89,9 @@
 		  :accessor link-href-end)
    (link-ampersand :type (or string null) :initform nil :initarg :link-ampersand
 		   :accessor link-ampersand))
-  (:default-initargs :link-page "lookup-func1")
+  (:default-initargs :link-page "meta-search.html")
   (:documentation "View class for a hyperobject"))
 
-
 (defun get-default-view-id (obj-cl)
   (aif (views obj-cl)
       (id (car it))
@@ -237,6 +236,11 @@
   (vector-push-extend `(write-string ,tag s) print-func)
   (vector-push-extend '(write-char #\> s) print-func))
 
+(defun ppfc-display-table (title name type formatter cdata print-func)
+  (vector-push-extend '(write-string "<td>" s) print-func)
+  (ppfc-html title name type formatter cdata print-func)
+  (vector-push-extend '(write-string "</td>" s) print-func))
+		      
 (defun ppfc-html-labels (label name type formatter cdata print-func)
   (vector-push-extend '(write-string "<span class=\"label\">" s) print-func)
   (vector-push-extend `(write-string ,label s) print-func)
@@ -304,6 +308,8 @@
        (ppfc-html-labels user-name name type formatter cdata print-func))
       (:xhtml-labels
        (ppfc-xhtml-labels xml-user-name user-name name type formatter cdata print-func))
+      ((:display-table :display-table-labels)
+       (ppfc-display-table user-name name type formatter cdata print-func))
       (:xml-labels
        (ppfc-xml-labels xml-user-name xml-tag name type formatter cdata print-func))
       ((or :html-link :xhtml-link)
@@ -347,7 +353,7 @@
       :xhtml-link :html-link
       :xml :xml-labels :xml-link :ie-xml-link
       :xml-link-labels :ie-xml-link-labels
-      :display-table :edit-table))
+      :display-table :display-table-labels :edit-table :edit-table-labels))
 
 (defun initialize-view-by-id (obj-cl view)
   "Initialize a view based upon a preset vid"
@@ -357,7 +363,7 @@
   (unless (slots view) (setf (slots view) (default-print-slots obj-cl)))
 
   (let ((links '())
-	(print-func (make-array 10 :fill-pointer 0 :adjustable t)))
+	(print-func (make-array 20 :fill-pointer 0 :adjustable t)))
 
     (do* ((slots (slots view) (cdr slots))
 	  (slot-name (car slots) (car slots))
@@ -406,6 +412,13 @@
      (setf (link-href-start view) "a href=")
      (setf (link-href-end view) "a")
      (setf (link-ampersand view) "&amp;"))
+    ((or :display-table :display-table-labels :edit-tables)
+     (initialize-table-view view)
+     (when (in (id view) :display-table-labels :edit-table-labels)
+       (setf (list-start-printer view) #'table-label-list-start-func))
+     (setf (link-href-start view) "a href=")
+     (setf (link-href-end view) "a")
+     (setf (link-ampersand view) "&amp;"))
     ((or :xml-link :xml-link-labels)
      (initialize-xml-view view)
      (setf (link-href-start view)
@@ -449,7 +462,7 @@
   (setf (indenter view) #'indent-spaces))
 
 (defun html-list-start-func (obj nitems indent strm)
-  (write-string "<div class=\"ho-username\" :style=\"margin-left:" strm)
+  (write-string "<div class=\"ho-username\" style=\"margin-left:" strm)
   (write-fixnum (+ indent indent) strm)
   (write-string "em;\">" strm)
   (write-user-name-maybe-plural obj nitems strm)
@@ -475,16 +488,49 @@
   (setf (obj-data-indent view) nil))
 
 (defun xhtml-list-start-func (obj nitems indent strm)
-  (write-string "<div class=\"ho-username\" :style=\"margin-left:" strm)
+  (write-string "<div class=\"ho-username\" style=\"margin-left:" strm)
   (write-fixnum (+ indent indent) strm)
   (write-string "em;\">" strm)
   (write-user-name-maybe-plural obj nitems strm)
   (write-string "</div>" strm)
-  (write-string "<div :style=\"margin-left:" strm)
+  (write-string "<div style=\"margin-left:" strm)
   (write-fixnum (+ indent indent) strm)
   (write-string "em;\">" strm)
   (write-char #\newline strm))
 
+(defun table-list-start-func (obj nitems indent strm)
+  (write-string "<div class=\"ho-username\" style=\"margin-left:" strm)
+  (write-fixnum (+ indent indent) strm)
+  (write-string "em;\">" strm)
+  (write-user-name-maybe-plural obj nitems strm)
+  (write-string "</div>" strm)
+  (write-char #\newline strm)
+  (write-string "<table style=\"margin-left:" strm)
+  (write-fixnum (+ indent indent) strm)
+  (write-string "em;\">" strm)
+  (write-string "<tbody>" strm)
+  (write-char #\newline strm))
+
+(defun table-label-list-start-func (obj nitems indent strm)
+  (write-string "<div class=\"ho-username\" style=\"margin-left:" strm)
+  (write-fixnum (+ indent indent) strm)
+  (write-string "em;\">" strm)
+  (write-user-name-maybe-plural obj nitems strm)
+  (write-string "</div>" strm)
+  (write-char #\newline strm)
+  (write-string "<table style=\"margin-left:" strm)
+  (write-fixnum (+ indent indent) strm)
+  (write-string "em;\">" strm)
+  (write-string "<thead>" strm)
+  (dolist (slot (default-print-slots (class-of obj)))
+    (write-string "<th>" strm)
+    (write-string (write-to-string slot) strm)
+    (write-string "</th>" strm))
+  (write-string "</thead>" strm)
+  (write-char #\newline strm)
+  (write-string "<tbody>" strm)
+  (write-char #\newline strm))
+
 (defun html-obj-start (obj indent strm)
   (declare (ignore obj indent))
   (write-string "<div style=\"margin-left:2em;\">" strm))
@@ -503,6 +549,21 @@
   (setf (obj-end-printer view) (format nil "</div>~%"))
   (setf (obj-data-indent view) nil))
 
+(defun initialize-table-view (view)
+  (initialize-text-view view)
+  (setf (indenter view) #'indent-spaces)
+  (setf (file-start-str view) (format nil "<html><body>~%"))
+  (setf (file-end-str view) (format nil "</body><html>~%"))
+  (setf (list-start-indent view) nil)
+  (setf (list-start-printer view) #'table-list-start-func)
+  (setf (list-end-printer view) (format nil "</tbody>~%</table>~%"))
+  (setf (list-end-indent view) nil)
+  (setf (obj-start-indent view) nil)
+  (setf (obj-start-printer view) #'html-obj-start)
+  (setf (obj-start-printer view) (format nil "<tr>"))
+  (setf (obj-end-printer view) (format nil "</tr>~%"))
+  (setf (obj-data-indent view) nil))
+
 (defun xmlformat-list-end-func (x strm)
   (write-string "</" strm)
   (write-string (class-name-of x) strm)
@@ -528,6 +589,7 @@
     (setf (list-end-printer view) #'xmlformat-list-end-func)
     (setf (obj-start-printer view) (format nil "<~(~a~)>" name))
     (setf (obj-start-indent view) t)
+    (setf (obj-end-printer view) (format nil "</~(~a~)>~%" name))
     (setf (subobj-end-printer view) (format nil "</~(~a~)>~%" name))
     (setf (subobj-end-indent view) nil)
     (setf (obj-data-indent view) nil)))
@@ -710,11 +772,13 @@
 	  (fmt-obj-start obj view strm indent)
 	  (fmt-obj-data obj view strm (1+ indent) refvars link-printer)
 	  (fmt-obj-end obj view strm indent)
-	  (when subobjects 
-	    (fmt-subobj-start obj view strm indent)
-	    (view-subobjects obj strm vid indent filter subobjects
-			     refvars link-printer)
-	    (fmt-subobj-end obj view strm indent))))
+	  (if subobjects 
+	      (progn
+		(fmt-subobj-start obj view strm indent)
+		(view-subobjects obj strm vid indent filter subobjects
+				 refvars link-printer)
+		(fmt-subobj-end obj view strm indent))
+	    (fmt-subobj-start obj view strm indent))))
       (fmt-list-end (car objlist) view strm indent nobjs)))
   objs)
 
